#ifndef __COMMON_H__
#define __COMMON_H__

/*
  QtCurve (C) Craig Drummond, 2003 Craig.Drummond@lycos.co.uk

----

  Released under the GNU General Public License (GPL) v2.

----

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files(the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

/* This file is quite 'hacky' as it contains lots of static function definitions - but I want to share the code
   between Qt and Gtk, but not polute the namespace with exported functions... */

#include <string.h>
#include <math.h>

#ifdef __cplusplus
#include <qstring.h>
#include <qsettings.h>
#ifdef QTC_CONFIG_DIALOG
#include <qapplication.h>
#endif
#else
#include <glib.h>
#endif

#define QTC_GROUP     "Settings"
#define QTC_GROUP_LEN 8

#define QTC_CHECK_SIZE   13
#define QTC_RADIO_SIZE   13
#define QTC_MIN_BTN_SIZE 8
#define QTC_LV_SIZE(X)   LV_EXP_ARR==(X) ? 7 : 5

#define NUM_SHADES 7

/* Progressbar and selected menu items... */
#define SHADE_GRADIENT_TOP         0.92
#define SHADE_GRADIENT_BOTTOM      1.66
#define SHADE_GRADIENT_LIGHT       1.62
#define SHADE_GRADIENT_DARK        1.05

/* 3d effect - i.e. buttons, etc */
#define QTC_SHADES \
    static double shades[2][11][NUM_SHADES]=\
    { \
        { \
          { 1.01, 1.03, 0.93, 0.780, 0.752, 0.74, 1.8 }, \
          { 1.02, 1.03, 0.93, 0.781, 0.753, 0.70, 1.8 }, \
          { 1.03, 1.03, 0.93, 0.782, 0.754, 0.68, 1.8 }, \
          { 1.04, 1.04, 0.93, 0.783, 0.755, 0.64, 1.8 }, \
          { 1.05, 1.04, 0.93, 0.784, 0.756, 0.62, 1.8 }, \
          { 1.06, 1.05, 0.93, 0.785, 0.758, 0.58, 1.8 }, \
          { 1.07, 1.05, 0.93, 0.786, 0.760, 0.54, 1.8 }, \
          { 1.09, 1.06, 0.93, 0.787, 0.762, 0.50, 1.8 }, /* default */ \
          { 1.11, 1.06, 0.93, 0.788, 0.764, 0.45, 1.8 }, \
          { 1.13, 1.07, 0.93, 0.789, 0.766, 0.40, 1.8 }, \
          { 1.15, 1.07, 0.93, 0.790, 0.768, 0.35, 1.8 } \
        }, \
        { \
          { 1.08, 1.03, 0.93, 0.780, 0.832, 0.74, 1.8 }, \
          { 1.09, 1.03, 0.93, 0.781, 0.833, 0.72, 1.8 }, \
          { 1.10, 1.03, 0.93, 0.782, 0.834, 0.70, 1.8 }, \
          { 1.11, 1.04, 0.93, 0.783, 0.835, 0.68, 1.8 }, \
          { 1.12, 1.04, 0.93, 0.784, 0.836, 0.66, 1.8 }, \
          { 1.13, 1.05, 0.93, 0.785, 0.838, 0.64, 1.8 }, \
          { 1.14, 1.05, 0.93, 0.786, 0.840, 0.62, 1.8 }, \
          { 1.15, 1.06, 0.93, 0.787, 0.842, 0.60, 1.8 }, /* default */ \
          { 1.17, 1.06, 0.93, 0.788, 0.844, 0.45, 1.8 }, \
          { 1.19, 1.07, 0.93, 0.789, 0.846, 0.40, 1.8 }, \
          { 1.21, 1.07, 0.93, 0.790, 0.848, 0.35, 1.8 } \
        } \
    };
#if 0 /* Old, Pre 0.33 light gradient shades... */
        { \
          { 1.08, 1.03, 0.93, 0.780, 0.800, 0.74, 1.8 }, \
          { 1.09, 1.03, 0.93, 0.781, 0.810, 0.72, 1.8 }, \
          { 1.10, 1.03, 0.93, 0.782, 0.820, 0.70, 1.8 }, \
          { 1.11, 1.04, 0.93, 0.783, 0.840, 0.68, 1.8 }, \
          { 1.12, 1.04, 0.93, 0.784, 0.860, 0.66, 1.8 }, \
          { 1.13, 1.05, 0.93, 0.785, 0.880, 0.64, 1.8 }, \
          { 1.14, 1.05, 0.93, 0.786, 0.900, 0.62, 1.8 }, \
          { 1.15, 1.06, 0.93, 0.787, 0.920, 0.60, 1.8 }, /* default */ \
          { 1.17, 1.06, 0.93, 0.788, 0.764, 0.45, 1.8 }, \
          { 1.19, 1.07, 0.93, 0.789, 0.766, 0.40, 1.8 }, \
          { 1.21, 1.07, 0.93, 0.790, 0.768, 0.35, 1.8 } \
        } \
    };
#endif

#define QTC_SHADE(a, c, s) \
    (c>10 || c<0 || s>=NUM_SHADES || s<0 ? 1.0 : shades[!LIGHT_APPEARANCE(a) ? 0 : 1][c][s])

/* Shades used when gradient effect is selected */
#define LIGHT_APPEARANCE(A) (APPEARANCE_LIGHT_GRADIENT==(A) || \
                             APPEARANCE_BEVELLED==(A) || \
                             IS_GLASS(A) || \
                             APPEARANCE_INVERTED==(A))

#define TAB_APPEARANCE(A)   (A) /* (APPEARANCE_GLASS==(A) ? APPEARANCE_LIGHT_GRADIENT : (A)) */

#define SAME_NUM_PROFILE_COLS(A, B)   ((PROFILE_RAISED_GRADIENT==A || PROFILE_RAISED_LIGHT_GRADIENT==A) && \
                                       (PROFILE_RAISED_GRADIENT==B || PROFILE_RAISED_LIGHT_GRADIENT==B))

#define SHADE_BEVEL_GRAD_LIGHT(A)     ((!LIGHT_APPEARANCE(A)) ? 1.1 : 1.05 )
#define SHADE_BEVEL_GRAD_DARK(A)      ((!LIGHT_APPEARANCE(A)) ? 0.9 : 0.93 )
#define SHADE_BEVEL_GRAD_SEL_LIGHT(A) ((!LIGHT_APPEARANCE(A)) ? 1.1 : 1.05 )
#define SHADE_BEVEL_GRAD_SEL_DARK(A)  ((!LIGHT_APPEARANCE(A)) ? 0.95 : 0.95 )

#define SHADE_MENU_LIGHT(A)           ((!LIGHT_APPEARANCE(A)) ? 1.04 : 1.02 )
#define SHADE_MENU_DARK(A)            ((!LIGHT_APPEARANCE(A)) ? 0.94 : 0.96 )

#define SHADE_TAB_LIGHT(A)            ((!LIGHT_APPEARANCE(A)) ? 1.1 : 1.0 )
#define SHADE_TAB_DARK(A)             ((!LIGHT_APPEARANCE(A)) ? 1.0 : 1.0 )
#define SHADE_TAB_SEL_LIGHT(A)        ((!LIGHT_APPEARANCE(A)) ? 1.1 : 1.1 )
#define SHADE_TAB_SEL_DARK(A)         ((!LIGHT_APPEARANCE(A)) ? 1.0 : 1.0 )

#define SHADE_BOTTOM_TAB_LIGHT(A)     ((!LIGHT_APPEARANCE(A)) ? 1.0 : 1.0 )
#define SHADE_BOTTOM_TAB_DARK(A)      ((!LIGHT_APPEARANCE(A)) ? 0.95 : 1.0 )
#define SHADE_BOTTOM_TAB_SEL_LIGHT(A) 1.0 /*((!LIGHT_APPEARANCE(A)) ? 1.0 : 1.0 )*/
#define SHADE_BOTTOM_TAB_SEL_DARK(A)  0.9 /*((!LIGHT_APPEARANCE(A)) ? 0.92 : 0.92 ) */

#define SHADE_GLASS_TOP_A(A) (APPEARANCE_DULL_GLASS==A ? 1.20 : 1.7)
#define SHADE_GLASS_TOP_B(A) 0.96
#define SHADE_GLASS_BOT_A(A) 0.99
#define SHADE_GLASS_BOT_B(A) (APPEARANCE_DULL_GLASS==A ? 1.05 : 1.16)

#define IS_GLASS(A) (APPEARANCE_DULL_GLASS==A || APPEARANCE_SHINY_GLASS==A)

#define SHADE_SELECTION_TOP 1.15
#define SHADE_SELECTION_BOT 0.9

#ifdef __cplusplus
#define MENUBAR_DARK_LIMIT 160
#define TOO_DARK(A) ((A).red()<MENUBAR_DARK_LIMIT || (A).green()<MENUBAR_DARK_LIMIT || (A).blue()<MENUBAR_DARK_LIMIT)
#else
#define MENUBAR_DARK_LIMIT (160<<8)
#define TOO_DARK(A) ((A).red<MENUBAR_DARK_LIMIT || (A).green<MENUBAR_DARK_LIMIT || (A).blue<MENUBAR_DARK_LIMIT)
#endif

#define MENUBAR_SHADE_LIGHT_FACTOR 1.20
#define HIGHLIGHT_FACTOR           1.05
#define MENUBAR_DARK_FACTOR        0.95
#define POPUPMENU_LIGHT_FACTOR     1.15

#define USE_SHADED_MENU_BAR_COLORS (SHADE_CUSTOM==opts.shadeMenubars || SHADE_SELECTED==opts.shadeMenubars)
#define MENUBAR_GLASS_SELECTED_DARK_FACTOR 0.9
#define GLASS_MAX_PIXELS 12
#define GLASS_BORDER     0.4
#define BEVEL_BORDER(w)    (WIDGET_LISTVIEW_HEADER==w ? 6 : 4)
#define SHADE_BEVEL_TOP    1.07
#define SHADE_BEVEL_MID_TOP    1.01
#define SHADE_BEVEL_MID_BOT    0.975
#define SHADE_BEVEL_BOT(w) (WIDGET_LISTVIEW_HEADER==(w) ? 0.86 : 0.90)

typedef enum
{
    WIDGET_TAB_TOP,
    WIDGET_TAB_BOT,
    WIDGET_BUTTON,
    WIDGET_LISTVIEW_HEADER,
    WIDGET_OTHER
} EWidget;

typedef enum
{
    ROUNDED_NONE,
    ROUNDED_TOP,
    ROUNDED_BOTTOM,
    ROUNDED_LEFT,
    ROUNDED_RIGHT,

    ROUNDED_TOPRIGHT,
    ROUNDED_BOTTOMRIGHT,

    ROUNDED_ALL
} ERound;

typedef enum
{
    GRADIENT_BASE,
    GRADIENT_LIGHT,
    GRADIENT_DARK,
    GRADIENT_TOP,
    GRADIENT_BOTTOM,
    GRADIENT_NUM_COLS
} EGradients;

typedef enum
{
    IND_CORNER,
    IND_FONT_COLOR,
    IND_NONE
} EDefBtnIndicator;

typedef enum
{
    GROOVE_RAISED,
    GROOVE_SUNKEN,
    GROOVE_NONE
} EGroove;

typedef enum
{
    HANDLE_RAISED,
    HANDLE_SUNKEN,
    HANDLE_DOTS,
    HANDLE_DASHES
} EHandle;

typedef enum
{
    TB_NONE,
    TB_LIGHT,
    TB_DARK
} ETBarBorder;

typedef enum
{
    LV_EXP_PM,
    LV_EXP_ARR
} ELvExpander;

typedef enum
{
    LV_LINES_NONE,
    LV_LINES_DOTTED,
    LV_LINES_SOLID
} ELvLines;

typedef enum
{
    PROFILE_FLAT,
    PROFILE_RAISED_GRADIENT,
    PROFILE_SUNKEN_GRADIENT,
    PROFILE_RAISED_LIGHT_GRADIENT
} EProfile;

typedef enum
{
    APPEARANCE_FLAT,
    APPEARANCE_DULL_GLASS,
    APPEARANCE_SHINY_GLASS,
    APPEARANCE_GRADIENT,
    APPEARANCE_INVERTED,
    APPEARANCE_LIGHT_GRADIENT,
    APPEARANCE_BEVELLED
} EAppearance;

typedef enum
{
    SHADE_NONE,
    SHADE_SELECTED,
    SHADE_CUSTOM,
    SHADE_DARKEN    /* For menubar only! */
} EShade;

typedef enum
{
    HEADER_COLOR_BGND,
    HEADER_COLOR_BTN,
    HEADER_COLOR_CUSTOM
} EHeaderColor;

typedef enum
{
    SEPARATOR_DOTS,
    SEPARATOR_LINES,
    SEPARATOR_NONE
} ESeparator;

typedef enum
{
    FOCUS_DOTS,
    FOCUS_COLORED
} EFocus;

#define DEF_IND_STR                "fontcolor"
#define DEF_HANDLE_STR             "dots"
#define DEF_TB_BORDER              "none"
#define DEF_APPEARANCE_STR         "bevelled"
#define DEF_MENU_APPEARANCE_STR    "gradient"
#define DEF_TOOLBAR_APPEARANCE_STR "gradient"
#define DEF_PROFILE_STR            "lightraised"
#define DEF_SLIDER_SHADE_STR       "selected"
#define DEF_TBS_STR                "dots"
#define DEF_COLOR_STR              "background"
#define DEF_FOCUS_STR              "colored"
#define DEF_TOOLBAR_SHADE_STR      "none"

/*#define DEF_MENU_SHADE_STR         "darken"*/

#ifdef __cplusplus
typedef bool   qtc_bool;
typedef QColor qtc_color;
#else
typedef gboolean qtc_bool;
typedef GdkColor qtc_color;
#endif

#ifndef QTC_COMMON_NO_FUNCTIONS

#ifndef QTC_COMMON_ONLY_COVERTERS
#define QTC_MIN(a, b) ((a) < (b) ? (a) : (b))
#define QTC_MAX(a, b) ((b) < (a) ? (a) : (b))

static void qtc_rgb2hls(double *r, double *g, double *b)
{
    double min,
           max,
           red=*r,
           green=*g,
           blue=*b,
           h=0,
           l,
           s=0,
           delta;

    if(red > green)
    {
        max=QTC_MAX(red, blue);
        min=QTC_MIN(green, blue);
    }
    else
    {
        max=QTC_MAX(green, blue);
        min=QTC_MIN(red, blue);
    }

    l=(max +min)/ 2;
    s=0;
    h=0;

    if(max !=min)
    {
        if(l <=0.5)
            s=(max-min)/(max +min);
        else
            s=(max-min)/(2-max-min);

        delta=max-min;

        if(red==max)
            h=(green-blue)/ delta;
        else if(green==max)
            h=2 +(blue-red)/ delta;
        else if(blue==max)
            h=4 +(red-green)/ delta;

        h *=60;
        if(h < 0.0)
            h +=360;
    }

    *r=h;
    *g=l;
    *b=s;
}

static void qtc_hls2rgb(double *h, double *l, double *s)
{
    double hue,
           lightness=*l,
           saturation=*s,
           m1,
           m2,
           r,
           g,
           b;

    if(lightness <=0.5)
        m2=lightness *(1 +saturation);
    else
        m2=lightness +saturation-lightness * saturation;
    m1=2 * lightness-m2;

    if(0==saturation)
    {
        *h=lightness;
        *l=lightness;
        *s=lightness;
    }
    else
    {
        hue=*h +120;
        while(hue > 360)
            hue-=360;
        while(hue < 0)
            hue +=360;

        if(hue < 60)
            r=m1 +(m2-m1)* hue / 60;
        else if(hue < 180)
            r=m2;
        else if(hue < 240)
            r=m1 +(m2-m1)*(240-hue)/ 60;
        else
            r=m1;

        hue=*h;
        while(hue > 360)
            hue-=360;
        while(hue < 0)
            hue +=360;

        if(hue < 60)
            g=m1 +(m2-m1)* hue / 60;
        else if(hue < 180)
            g=m2;
        else if(hue < 240)
            g=m1 +(m2-m1)*(240-hue)/ 60;
        else
            g=m1;

        hue=*h-120;
        while(hue > 360)
            hue-=360;
        while(hue < 0)
            hue +=360;

        if(hue < 60)
            b=m1 +(m2-m1)* hue / 60;
        else if(hue < 180)
            b=m2;
        else if(hue < 240)
            b=m1 +(m2-m1)*(240-hue)/ 60;
        else
            b=m1;

        *h=r;
        *l=g;
        *s=b;
    }
}

static qtc_bool qtc_equal(double d1, double d2)
{
    return (fabs(d1 - d2) < 0.0001);
}

#endif

static EDefBtnIndicator qtc_to_ind(const char *str)
{
    if(0==memcmp(str, "fontcolor", 9) || 0==memcmp(str, "fontcolour", 10) || 0==memcmp(str, "border", 6))
        return IND_FONT_COLOR;
    if(0==memcmp(str, "none", 4))
        return IND_NONE;
    return IND_CORNER;
}

static EHandle qtc_to_handle(const char *str)
{
    if(0==memcmp(str, "raised", 6))
        return HANDLE_RAISED;
/*
    if(0==memcmp(str, "dots", 4))
        return HANDLE_DOTS;
*/
    if(0==memcmp(str, "dashes", 6))
        return HANDLE_DASHES;
/*
    return HANDLE_SUNKEN;
*/

    if(0==memcmp(str, "sunken", 6))
        return HANDLE_SUNKEN;
    return HANDLE_DOTS;
}

static EGroove qtc_to_groove(const char *str)
{
    if(0==memcmp(str, "raised", 6))
        return GROOVE_RAISED;
    if(0==memcmp(str, "none", 4))
        return GROOVE_NONE;
    return GROOVE_SUNKEN;
}

static ETBarBorder qtc_to_tbar_border(const char *str)
{
    if(0==memcmp(str, "dark", 4))
        return TB_DARK;
    if(0==memcmp(str, "none", 4))
        return TB_NONE;
    return TB_LIGHT;
}

static ELvExpander qtc_to_lv_expander(const char *str)
{
    return 0==memcmp(str, "arrow", 5) ? LV_EXP_ARR : LV_EXP_PM;
}

static ELvLines qtc_to_lv_lines(const char *str)
{
    if(0==memcmp(str, "solid", 5))
        return LV_LINES_SOLID;
    if(0==memcmp(str, "dotted", 6))
        return LV_LINES_DOTTED;
    return LV_LINES_NONE;
}

static EProfile qtc_to_profile(const char *str)
{
    if(0==memcmp(str, "flat", 4))
        return PROFILE_FLAT;
    if(0==memcmp(str, "raised", 6))
        return PROFILE_RAISED_GRADIENT;
    if(0==memcmp(str, "sunken", 6))
        return PROFILE_SUNKEN_GRADIENT;
    return PROFILE_RAISED_LIGHT_GRADIENT;
}

static EAppearance qtc_to_appearance(const char *str)
{
    if(0==memcmp(str, "flat", 4))
        return APPEARANCE_FLAT;
    if(0==memcmp(str, "gradient", 8))
        return APPEARANCE_GRADIENT;
    if(0==memcmp(str, "glass", 5) || 0==memcmp(str, "shinyglass", 10))
        return APPEARANCE_SHINY_GLASS;
    if(0==memcmp(str, "dullglass", 9))
        return APPEARANCE_DULL_GLASS;
/*
    if(0==memcmp(str, "bevelled", 8))
        return APPEARANCE_BEVELLED;
    return APPEARANCE_LIGHT_GRADIENT;
*/
    if(0==memcmp(str, "lightgradient", 13))
        return APPEARANCE_LIGHT_GRADIENT;
    if(0==memcmp(str, "inverted", 8))
        return APPEARANCE_INVERTED; 
    return APPEARANCE_BEVELLED;
}

static EShade qtc_to_shade(const char *str, qtc_bool allowDarken)
{
    /* true/false is from 0.25... */
    if(0==memcmp(str, "true", 4) || 0==memcmp(str, "TRUE", 4) || 0==memcmp(str, "selected", 8))
        return SHADE_SELECTED;
    if(allowDarken && 0==memcmp(str, "darken", 6))
        return SHADE_DARKEN;
    if(0==memcmp(str, "custom", 6))
        return SHADE_CUSTOM;
    return SHADE_NONE;
}

static ESeparator qtc_to_separator(const char *str)
{
    if(0==memcmp(str, "none", 4))
        return SEPARATOR_NONE;
/*
    if(0==memcmp(str, "dots", 4))
        return SEPARATOR_DOTS;
    return SEPARATOR_LINES!;
*/
    if(0==memcmp(str, "lines", 5))
        return SEPARATOR_LINES;
    return SEPARATOR_DOTS;
}

static EHeaderColor qtc_to_header_color(const char *str)
{
    if(0==memcmp(str, "background", 10))
        return HEADER_COLOR_BGND;
    if(0==memcmp(str, "custom", 6))
        return HEADER_COLOR_CUSTOM;
    return HEADER_COLOR_BTN;
}

static EFocus qtc_to_focus(const char *str)
{
    if(0==memcmp(str, "dots", 4))
        return FOCUS_DOTS;
    return FOCUS_COLORED;
}

#endif

#ifdef __cplusplus
struct Options
#else
typedef struct
#endif
{
#ifdef __cplusplus

#ifdef QTC_COMMON_NO_FUNCTIONS
    Options();
#else
#ifdef QTC_CONFIG_DIALOG
    Options() { }

    void loadSettings()
    {
#else
    Options()
    {
#endif
        QSettings s;

        contrast=s.readNumEntry("/Qt/KDE/contrast", 7);
        if(contrast<0 || contrast>10)
            contrast=7;

        focus=qtc_to_focus(s.readEntry("/qtcurvestyle/"QTC_GROUP"/focus", DEF_FOCUS_STR).latin1());
        border=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/border", true);
        bool darkerMenubarBgnd=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/darkerMenubarBgnd", true);
        lighterPopupMenuBgnd=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/lighterPopupMenuBgnd", true);
        round=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/round", true);
        highlightEdits=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/highlightEdits", true);
        menubarRoundTopOnly=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/menubarRoundTopOnly", true);
        /*etched=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/etched", false);*/
        QString tmp=s.readEntry("/qtcurvestyle/"QTC_GROUP"/toolbarBorders", QString::null);
        toolbarBorders=tmp.isEmpty()
                        ? TB_NONE
                        : qtc_to_tbar_border(tmp.latin1());
        tmp=s.readEntry("/qtcurvestyle/"QTC_GROUP"/sliderThumbs", QString::null);
        sliderThumbs=tmp.isEmpty()
                        ? s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/etched", true)
                              ? GROOVE_SUNKEN : GROOVE_RAISED
                        : qtc_to_groove(tmp.latin1());
        tmp=s.readEntry("/qtcurvestyle/"QTC_GROUP"/lvExpander", QString::null);
        lvExpander=tmp.isEmpty()
                        ? LV_EXP_ARR
                        : qtc_to_lv_expander(tmp.latin1());
        tmp=s.readEntry("/qtcurvestyle/"QTC_GROUP"/lvLines", QString::null);
        lvLines=tmp.isEmpty()
                        ? LV_LINES_NONE
                        : qtc_to_lv_lines(tmp.latin1());
        lvDark=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/lvDark", false);
        drawStatusBarFrames=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/drawStatusBarFrames", false);
        lvHeaderColor=qtc_to_header_color(s.readEntry("/qtcurvestyle/"QTC_GROUP"/lvHeaderColor",
                                                      DEF_COLOR_STR).latin1());
        customLvHeaderColor=QColor(s.readEntry("/qtcurvestyle/"QTC_GROUP"/customLvHeaderColor",
                                               "#000000"));

        if(HEADER_COLOR_CUSTOM==lvHeaderColor && Qt::black==customLvHeaderColor)
        {
            lvHeaderColor=HEADER_COLOR_BGND;
#ifdef QTC_CONFIG_DIALOG
            customLvHeaderColor=::QApplication::palette().active().background();
#endif
        }
        handles=qtc_to_handle(s.readEntry("/qtcurvestyle/"QTC_GROUP"/handles",
                                          DEF_HANDLE_STR).latin1());
        tmp=s.readEntry("/qtcurvestyle/"QTC_GROUP"/appearance", DEF_APPEARANCE_STR);
        appearance=qtc_to_appearance(tmp.latin1());
        tmp=s.readEntry("/qtcurvestyle/"QTC_GROUP"/lvHeaderAppearance", QString::null);
        lvHeaderAppearance=tmp.isEmpty() ? appearance : qtc_to_appearance(tmp.latin1());
        menubarAppearance=qtc_to_appearance(s.readEntry("/qtcurvestyle/"QTC_GROUP"/menubarAppearance",
                                            DEF_MENU_APPEARANCE_STR).latin1());
        if(APPEARANCE_LIGHT_GRADIENT==menubarAppearance || APPEARANCE_BEVELLED==menubarAppearance)
            menubarAppearance=APPEARANCE_GRADIENT;
        toolbarAppearance=qtc_to_appearance(s.readEntry("/qtcurvestyle/"QTC_GROUP"/toolbarAppearance",
                                            DEF_TOOLBAR_APPEARANCE_STR).latin1());
        if(APPEARANCE_LIGHT_GRADIENT==toolbarAppearance || APPEARANCE_BEVELLED==toolbarAppearance)
            toolbarAppearance=APPEARANCE_GRADIENT;
        tmp=s.readEntry("/qtcurvestyle/"QTC_GROUP"/pm", DEF_PROFILE_STR).latin1();
        menuitemProfile=qtc_to_profile(s.readEntry("/qtcurvestyle/"QTC_GROUP"/menuitemProfile",
                                                   tmp).latin1());
        progressbarProfile=qtc_to_profile(s.readEntry("/qtcurvestyle/"QTC_GROUP"/progressbarProfile",
                                                      tmp).latin1());
        vArrow=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/vArrow", false);
        embolden=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/embolden", false);
        crLabelHighlight=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/crLabelHighlight", false);
        defBtnIndicator=qtc_to_ind(s.readEntry("/qtcurvestyle/"QTC_GROUP"/defBtnIndicator",
                                               DEF_IND_STR).latin1());
        if(!embolden && IND_NONE==defBtnIndicator)
            defBtnIndicator=IND_FONT_COLOR;
        borderSplitter=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/borderSplitter", false);
        highlightTab=s.readBoolEntry("/qtcurvestyle/"QTC_GROUP"/highlightTab", true);

        toolbarSeparators=qtc_to_separator(s.readEntry("/qtcurvestyle/"QTC_GROUP"/toolbarSeparators",
                                           DEF_TBS_STR).latin1());
        shadeSliders=qtc_to_shade(s.readEntry("/qtcurvestyle/"QTC_GROUP"/shadeSliders",
                                  DEF_SLIDER_SHADE_STR).latin1(), false);
        customSlidersColor=QColor(s.readEntry("/qtcurvestyle/"QTC_GROUP"/customSlidersColor",
                                              "#000000"));

        if(SHADE_CUSTOM==shadeSliders && Qt::black==customSlidersColor)
        {
            shadeSliders=SHADE_NONE;
#ifdef QTC_CONFIG_DIALOG
            customSlidersColor=::QApplication::palette().active().background();
#endif
        }

        QString str(s.readEntry("/qtcurvestyle/"QTC_GROUP"/shadeMenubars", QString::null));

        if(str.isEmpty())
            shadeMenubars=darkerMenubarBgnd ? SHADE_DARKEN : SHADE_NONE;
        else
            shadeMenubars=qtc_to_shade(str.latin1(), true);

        customMenubarsColor=QColor(s.readEntry("/qtcurvestyle/"QTC_GROUP"/customMenubarsColor", "#000000"));

        if(SHADE_CUSTOM==shadeMenubars && Qt::black==customMenubarsColor)
        {
            shadeMenubars=SHADE_NONE;
#ifdef QTC_CONFIG_DIALOG
            customMenubarsColor=QApplication::palette().active().background();
#endif
        }
    };
#endif
#endif

    int              contrast;
    qtc_bool         border,
                     round,
                     vArrow,
                     embolden,
                     crLabelHighlight,
                     borderSplitter,
                     lighterPopupMenuBgnd,
                     highlightTab,
                     highlightEdits,
                     /*etched,*/
#ifdef __cplusplus
                     lvDark,
#endif
                     drawStatusBarFrames,
                     menubarRoundTopOnly;
    EFocus           focus;
    ETBarBorder      toolbarBorders;
    EDefBtnIndicator defBtnIndicator;
    EGroove          sliderThumbs;
    EHandle          handles;
    ELvExpander      lvExpander;
    ELvLines         lvLines;
    EHeaderColor     lvHeaderColor;
    EProfile         menuitemProfile,
                     progressbarProfile;
    EAppearance      appearance,
                     menubarAppearance,
                     toolbarAppearance,
                     lvHeaderAppearance;
    EShade           shadeSliders,
                     shadeMenubars;
    qtc_color        customMenubarsColor,
                     customSlidersColor,
                     customLvHeaderColor;
    ESeparator       toolbarSeparators;
#ifdef QTC_CONFIG_DIALOG
    QString          name,
                     file;
    bool             def,
                     readOnly;
#endif
#ifndef __cplusplus
} Options;

static void qtc_defaults(Options *opts)
{
    opts->contrast=7;
    opts->round=TRUE;
    opts->border=TRUE;
    opts->lighterPopupMenuBgnd=TRUE;
    opts->vArrow=FALSE;
    opts->highlightTab=TRUE;
    opts->highlightEdits=TRUE;
    opts->borderSplitter=FALSE;
    opts->crLabelHighlight=FALSE;
    opts->embolden=FALSE;
    opts->menubarRoundTopOnly=TRUE;
    opts->focus=FOCUS_COLORED;
    /*opts->etched=FALSE;*/
    /*opts->lvDark=FALSE; */
    opts->drawStatusBarFrames=FALSE;
    opts->menuitemProfile=PROFILE_RAISED_LIGHT_GRADIENT;
    opts->progressbarProfile=PROFILE_RAISED_LIGHT_GRADIENT;
    opts->appearance=APPEARANCE_BEVELLED;
    opts->menubarAppearance=APPEARANCE_GRADIENT;
    opts->toolbarAppearance=APPEARANCE_GRADIENT;
    opts->defBtnIndicator=IND_FONT_COLOR;
    opts->sliderThumbs=GROOVE_SUNKEN;
    opts->handles=HANDLE_DOTS;
    opts->shadeSliders=SHADE_SELECTED;
    opts->shadeMenubars=SHADE_DARKEN;
    opts->toolbarBorders=TB_NONE;
    opts->lvExpander=LV_EXP_ARR;
    opts->lvLines=LV_LINES_NONE;
    opts->lvHeaderAppearance=APPEARANCE_BEVELLED;
    opts->lvHeaderColor=HEADER_COLOR_BGND;
    opts->toolbarSeparators=SEPARATOR_DOTS;
    opts->customMenubarsColor.red=opts->customMenubarsColor.green=opts->customMenubarsColor.blue=0;
    opts->customSlidersColor.red=opts->customSlidersColor.green=opts->customSlidersColor.blue=0;
    opts->customLvHeaderColor.red=opts->customLvHeaderColor.green=opts->customLvHeaderColor.blue=0;
}

#else
};
#endif

#endif
