#ifndef __QTCURVE_H__
#define __QTCURVE_H__

/*
  QtCurve (C) Craig Drummond, 2003 Craig.Drummond@lycos.co.uk

  ----

  Based upon B???Curve style (Copyright (c) 2002 R?d H?t, Inc)
      Bernhard Rosenkrazer <bero@r?dh?t.com>
      Preston Brown <pbrown@r?dh?t.com>
      Than Ngo <than@r?dh?t.com>

  Released under the GNU General Public License (GPL) v2.

  ----

  B???Curve is based on the KDE Light style, 2nd revision:
  Copyright (c) 2000-2001 Trolltech AS (info@trolltech.com)

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

#include <kdeversion.h>
#include <kstyle.h>
#include <qcolor.h>
#include <qpoint.h>
#include <qpalette.h>
#include "common.h"

class HighContrastStyle : public KStyle
{
    Q_OBJECT

    public:

    HighContrastStyle();
    virtual ~HighContrastStyle() { }

    const char * className();
};

class QtCurveStyle : public HighContrastStyle
{
    Q_OBJECT

    public:

    enum EApp
    {
        APP_KICKER,
        APP_KORN,
        APP_OPENOFFICE,
        APP_MACTOR,
        APP_KPRESENTER,
        APP_OTHER
    };

    enum EHover
    {
        HOVER_NONE,
        HOVER_CHECK,
        HOVER_RADIO,
        HOVER_SB_ADD,
        HOVER_SB_SUB,
        HOVER_SB_SLIDER,
        HOVER_HEADER,
        HOVER_KICKER,
        HOVER_SW_UP,
        HOVER_SW_DOWN
    };

    QtCurveStyle();
    virtual ~QtCurveStyle();

    void polish(QApplication *app);
    void polish(QPalette &pal);
    void polish(QWidget *widget);
    void unPolish(QWidget *widget);
    void drawLightBevel(QPainter *p, const QRect &r, const QColorGroup &cg, QStyle::SFlags flags, bool useGrad,
                        ERound round, const QColor &fill, const QColor *custom=NULL, bool light=false,
                        const QColor *borderColor=NULL, bool doBorder=true, bool doCorners=true,
                        EWidget w=WIDGET_OTHER) const
    { drawLightBevel(cg.background(), p, r, cg, flags, useGrad, round, fill, custom, light, borderColor,
                     doBorder, doCorners, w); }
    void drawLightBevel(const QColor &bgnd, QPainter *p, const QRect &r, const QColorGroup &cg, QStyle::SFlags flags,
                        bool useGrad, ERound round, const QColor &fill, const QColor *custom=NULL,
                        bool light=false, const QColor *borderColor=NULL, bool doBorder=true, bool doCorners=true,
                        EWidget w=WIDGET_OTHER) const;
    void drawBorder(const QColor &bgnd, QPainter *p, const QRect &r, const QColorGroup &cg, QStyle::SFlags flags,
                    ERound round, const QColor *custom=NULL, const QColor *borderColor=NULL, bool doCorners=true) const;
    void drawEntryField(QPainter *p, const QRect &r, const QColorGroup &cg, QStyle::SFlags flags, bool highlight,
                        ERound round, bool isSpin=false) const;
    void drawArrow(QPainter *p, const QRect &r, const QColorGroup &cg, QStyle::SFlags flags,
                   QStyle::PrimitiveElement pe,  bool small=false, bool checkActive=false) const;
    void drawPrimitive(PrimitiveElement, QPainter *, const QRect &, const QColorGroup &, SFlags = Style_Default,
                       const QStyleOption & = QStyleOption::Default) const;
    void drawKStylePrimitive(KStylePrimitive kpe, QPainter* p, const QWidget* widget, const QRect &r,
                             const QColorGroup &cg, SFlags flags, const QStyleOption &opt ) const;
    void drawControl(ControlElement, QPainter *, const QWidget *, const QRect &, const QColorGroup &,
                     SFlags = Style_Default, const QStyleOption & = QStyleOption::Default) const;
    void drawControlMask(ControlElement, QPainter *, const QWidget *, const QRect &,
                         const QStyleOption & = QStyleOption::Default) const;
    void drawComplexControlMask(ComplexControl control, QPainter *p, const QWidget *widget, const QRect &r,
                                const QStyleOption &data) const;
    QRect subRect(SubRect, const QWidget *) const;
    void drawComplexControl(ComplexControl, QPainter *, const QWidget *, const QRect &, const QColorGroup &,
                            SFlags = Style_Default, SCFlags = SC_All, SCFlags = SC_None,
                            const QStyleOption & = QStyleOption::Default) const;
    QRect querySubControlMetrics(ComplexControl, const QWidget *, SubControl,
                                 const QStyleOption & = QStyleOption::Default) const;
    int pixelMetric(PixelMetric, const QWidget *widget= 0) const;
    int kPixelMetric(KStylePixelMetric kpm, const QWidget *widget) const;
    QSize sizeFromContents(ContentsType, const QWidget *, const QSize &,
                           const QStyleOption & = QStyleOption::Default) const;
    int styleHint(StyleHint, const QWidget *widget= 0, const QStyleOption & = QStyleOption::Default,
                  QStyleHintReturn *returnData= 0) const;

    protected:

    bool eventFilter(QObject *object, QEvent *event);
    void drawPBarOrMenu(QPainter *p, QRect const &r, bool horiz, const QColorGroup &cg, ControlElement control,
                        bool menu) const;
    void drawGradientWithBorder(QPainter *p, QRect const &r, bool horiz, const QColor *cols) const;
    void drawBevelGradient(const QColor &base, bool increase, int border, QPainter *p, QRect const &r, bool horiz,
                           double shadeTop, double shadeBot, bool sel, EAppearance bevApp, EWidget w=WIDGET_OTHER) const;
    void drawGradient(const QColor &top, const QColor &bot, bool increase, int border, QPainter *p, QRect const &r,
                      bool horiz=true) const;
    void drawSliderHandle(QPainter *p, const QRect &r, const QColorGroup &cg, QStyle::SFlags flags) const;
    void drawSliderGroove(QPainter *p, const QRect &r, SFlags flags, const QWidget *widget) const;
    void drawMenuOrToolBarBackground(QPainter *p, const QRect &r, const QColorGroup &cg, bool menu=true,
                                     bool horiz=true) const;

    private:

    void           shadeColors(const QColor &base, QColor *vals) const;
    const QColor * buttonColors(const QColorGroup &cg) const;
    const QColor * sliderColors(const QColorGroup &cg) const;
    const QColor * backgroundColors(const QColor &c) const;
    const QColor * backgroundColors(const QColorGroup &cg) const { return backgroundColors(cg.background()); }
    void           setMenuColors(const QColorGroup &cg);
    bool           redrawHoverWidget();

    private:

    QColor           menuitem[GRADIENT_NUM_COLS<NUM_SHADES+1 ? NUM_SHADES+1 : GRADIENT_NUM_COLS],
                     *progressbar,
                     gray[NUM_SHADES+1],
                     menubar[NUM_SHADES+1],
                     *highlightEntry,
                     *slider,
                     *listview,
                     button[NUM_SHADES+1],   // Last color = base color, for comparisons!
                     lighterPopupMenuBgndCol,
                     menubarColor,
                     focusColor;
    mutable QColor   buttonColored[NUM_SHADES+1];
    mutable QColor   backgroundColored[NUM_SHADES+1];
    EApp             themedApp;
    Options          opts;
#if KDE_VERSION >= 0x30200
    bool             isTransKicker;
#endif
    EHover           hover;
    int              contrast;
    QPoint           oldCursor;
    mutable bool     formMode;
    QWidget          *hoverWidget;
    int              hoverSect;
    QPalette         *mactorPal;
};

#endif
